@@ -4,13 +4,214 @@ import json
 from sklearn.pipeline import make_pipeline
 from river import compose
 
+from spotPython.utils.convert import class_for_name
+from spotPython.utils.transform import transform_hyper_parameter_values
 
-from spotPython.hyperparameters.prepare import (
-    transform_hyper_parameter_values,
-    get_dict_with_levels_and_types,
-    convert_keys,
-    iterate_dict_values,
-)
+
+def generate_one_config_from_var_dict(var_dict, fun_control) -> dict:
+    """Generate one configuration from a dictionary of variables (as a generator).
+    This function takes a dictionary of variables as input arguments and returns a dictionary
+    with the values from the arrays in the dictionary.
+    Args:
+        var_dict (dict): A dictionary where keys are variable names and values are numpy arrays.
+        fun_control (dict): A dictionary which (at least) has an entry with the following key:
+            - "var_type": A list of variable types. If the entry is not "num" the corresponding
+            value will be converted to the type "int".
+    Returns:
+        dict: A dictionary with the values from the arrays in the dictionary.
+    Example:
+        >>> import numpy as np
+        >>> var_dict = {'a': np.array([1, 3, 5]), 'b': np.array([2, 4, 6])}
+        >>> fun_control = {"var_type": ["int", "num"]}
+        >>> generate_one_config_from_var_dict(var_dict, fun_control)
+        {'a': 1, 'b': 2}
+    """
+    for values in iterate_dict_values(var_dict):
+        values = convert_keys(values, fun_control["var_type"])
+        values = get_dict_with_levels_and_types(fun_control=fun_control, v=values)
+        values = transform_hyper_parameter_values(fun_control=fun_control, hyper_parameter_values=values)
+        yield values
+
+
+def return_conf_list_from_var_dict(var_dict: dict, fun_control: dict) -> list:
+    """This function takes a dictionary of variables and a dictionary of function control.
+    It performs similar steps as generate_one_config_from_var_dict()
+    but returns a list of dictionaries of hyper parameter values.
+    Args:
+        var_dict (dict): A dictionary of variables.
+        fun_control (dict): A dictionary of function control.
+    Returns:
+        list A list of dictionaries of hyper parameter values. Transformations are applied to the values.
+    Examples:
+        >>> import numpy as np
+            var_dict = {'a': np.array([1]),
+                        'b': np.array([2])}
+            fun_control = {'var_type': ['int', 'int']}
+            return_conf_list_from_var_dict(var_dict, fun_control)
+            var_dict = {'a': np.array([1, 3, 5]), 'b': np.array([2, 4, 6])}
+            fun_control = {'var_type': ['int', 'int']}
+            return_conf_list_from_var_dict(var_dict, fun_control)
+            {'a': [1, 3, 5], 'b': [2, 4, 6]}
+
+    """
+    conf_list = []
+    for values in generate_one_config_from_var_dict(var_dict, fun_control):
+        conf_list.append(values)
+    return conf_list
+
+
+def iterate_dict_values(var_dict: dict):
+    """
+    This function takes a dictionary of variables as input arguments and returns an iterator that
+    yields the values from the arrays in the dictionary.
+
+    Parameters:
+        var_dict (dict): A dictionary where keys are variable names and values are numpy arrays.
+
+    Returns:
+        iterator: An iterator that yields the values from the arrays in the dictionary.
+
+    Example:
+        >>> import numpy as np
+        >>> var_dict = {'a': np.array([1, 3, 5]), 'b': np.array([2, 4, 6])}
+        >>> for values in iterate_dict_values(var_dict):
+        ...     print(values)
+        {'a': 1, 'b': 2}
+        {'a': 3, 'b': 4}
+        {'a': 5, 'b': 6}
+    """
+    n = len(next(iter(var_dict.values())))
+    for i in range(n):
+        yield {key: value[i] for key, value in var_dict.items()}
+
+
+def convert_keys(d: dict, var_type: list):
+    """
+    Convert values in a dictionary to integers based on a list of variable types.
+
+    This function takes a dictionary `d` and a list of variable types `var_type` as arguments.
+    For each key in the dictionary,
+    if the corresponding entry in `var_type` is not equal to `"num"`,
+    the value associated with that key is converted to an integer.
+
+    Args:
+        d (dict): The input dictionary.
+        var_type (list): A list of variable types. If the entry is not `"num"` the corresponding
+        value will be converted to the type `"int"`.
+
+    Returns:
+        dict: The modified dictionary with values converted to integers based on `var_type`.
+
+    Example:
+        >>> d = {'a': '1.1', 'b': '2', 'c': '3.1'}
+        >>> var_type = ["int", "num", "int"]
+        >>> convert_keys(d, var_type)
+        {'a': 1, 'b': '2', 'c': 3}
+    """
+    keys = list(d.keys())
+    for i in range(len(keys)):
+        if var_type[i] not in ["num", "float"]:
+            d[keys[i]] = int(d[keys[i]])
+    return d
+
+
+def get_dict_with_levels_and_types(fun_control, v) -> dict:
+    """Get dictionary with levels and types.
+    The function maps the numerical output of the hyperparameter optimization to the corresponding levels
+    of the hyperparameter needed by the core model, i.e., the tuned algorithm.
+    The function takes the dictionaries fun_control and v and returns a new dictionary with the same keys as v
+    but with the values of the levels of the keys from fun_control.
+    If the key value in the dictionary is 0, it takes the first value from the list,
+    if it is 1, it takes the second and so on.
+    If a key is not in fun_control, it takes the key from v.
+    If the core_model_parameter_type value is instance, it returns the class of the value from the module
+    via getattr("class", value).
+    For example,
+    if fun_control = {"HoeffdingTreeRegressor":{
+        "leaf_prediction": {
+            "levels": ["mean", "model", "adaptive"],
+            "type": "factor",
+            "default": "mean",
+            "core_model_parameter_type": "str"},
+        "leaf_model": {
+            "levels": ["linear_model.LinearRegression", "linear_model.PARegressor", "linear_model.Perceptron"],
+            "type": "factor",
+            "default": "LinearRegression",
+            "core_model_parameter_type": "instance"},
+            "splitter": {"levels": ["EBSTSplitter", "TEBSTSplitter", "QOSplitter"],
+            "type": "factor",
+            "default": "EBSTSplitter", "core_model_parameter_type": "instance()"},
+        "binary_split": {
+            "levels": [0, 1],
+            "type": "factor",
+            "default": 0,
+            "core_model_parameter_type": "bool"},
+        "stop_mem_management": {
+            "levels": [0, 1],
+            "type": "factor",
+            "default": 0,
+            "core_model_parameter_type": "bool"}}}
+        and
+        v = {'grace_period': 200,
+            'max_depth': 10,
+            'delta': 1e-07,
+            'tau': 0.05,
+            'leaf_prediction': 0,
+            'leaf_model': 0,
+            'model_selector_decay': 0.95,
+            'splitter': 1,
+            'min_samples_split': 9,
+            'binary_split': 0,
+            'max_size': 500.0}
+        then the function returns
+        {'grace_period': 200,
+        'max_depth': 10,
+        'delta': 1e-07,
+        'tau': 0.05,
+        'leaf_prediction': 'mean',
+        'leaf_model': linear_model.LinearRegression,
+        'model_selector_decay': 0.95,
+        'splitter': 'TEBSTSplitter',
+        'min_samples_split': 9,
+        'binary_split': 0,
+        'max_size': 500.0}.
+
+    Args:
+        fun_control (dict): dictionary with levels and types
+        v (dict): dictionary with values
+
+    Returns:
+        new_dict (dict): dictionary with levels and types
+
+    Example:
+        >>> fun_control = {"HoeffdingTreeRegressor":{
+                "leaf_prediction": {"levels": ["mean", "model", "adaptive"],
+                                    "type": "factor",
+                                    "default": "mean",
+                                    "core_model_parameter_type": "str"}}}
+            v = {"leaf_prediction": 0}
+            get_dict_with_levels_and_types(fun_control, v)
+            {"leaf_prediction": "mean"}
+    """
+    d = fun_control["core_model_hyper_dict"]
+    new_dict = {}
+    for key, value in v.items():
+        if key in d and d[key]["type"] == "factor":
+            if d[key]["core_model_parameter_type"] == "instance":
+                if "class_name" in d[key]:
+                    mdl = d[key]["class_name"]
+                c = d[key]["levels"][value]
+                new_dict[key] = class_for_name(mdl, c)
+            elif d[key]["core_model_parameter_type"] == "instance()":
+                mdl = d[key]["class_name"]
+                c = d[key]["levels"][value]
+                k = class_for_name(mdl, c)
+                new_dict[key] = k()
+            else:
+                new_dict[key] = d[key]["levels"][value]
+        else:
+            new_dict[key] = v[key]
+    return new_dict
 
 
 def assign_values(X: np.array, var_list: list) -> dict:
@@ -181,6 +382,52 @@ def get_var_type(fun_control) -> list:
     )
 
 
+def get_transform(fun_control) -> list:
+    """Get the transformations of the values from the dictionary fun_control as a list.
+    Args:
+        fun_control (dict): dictionary with levels and types
+    Returns:
+        (list): list with transformations
+    Example:
+        >>> d = {"core_model_hyper_dict":{
+            "leaf_prediction": {
+                "levels": ["mean", "model", "adaptive"],
+                "type": "factor",
+                "default": "mean",
+                "transform": "None",
+                "core_model_parameter_type": "str"},
+            "leaf_model": {
+                "levels": ["linear_model.LinearRegression", "linear_model.PARegressor", "linear_model.Perceptron"],
+                "type": "factor",
+                "default": "LinearRegression",
+                "transform": "None",
+                "core_model_parameter_type": "instance"},
+            "splitter": {
+                "levels": ["EBSTSplitter", "TEBSTSplitter", "QOSplitter"],
+                "type": "factor",
+                "default": "EBSTSplitter",
+                "transform": "None",
+                "core_model_parameter_type": "instance()"},
+            "binary_split": {
+                "levels": [0, 1],
+                "type": "factor",
+                "default": 0,
+                "transform": "None",
+                "core_model_parameter_type": "bool"},
+            "stop_mem_management": {                                                         "levels": [0, 1],
+                "type": "factor",
+                "default": 0,
+                "transform": "None",
+                "core_model_parameter_type": "bool"}}}
+
+        get_transform(d)
+        ['None', 'None', 'None', 'None', 'None']
+    """
+    return list(
+        fun_control["core_model_hyper_dict"][key]["transform"] for key in fun_control["core_model_hyper_dict"].keys()
+    )
+
+
 def get_var_name(fun_control) -> list:
     """Get the names of the values from the dictionary fun_control as a list.
     Args:
@@ -329,35 +576,7 @@ def get_values_from_dict(dictionary) -> np.array:
     return np.array(list(dictionary.values()))
 
 
-def return_conf_list_from_var_dict(var_dict: dict, fun_control: dict) -> list:
-    """This function takes a dictionary of variables and a dictionary of function control.
-    Args:
-        var_dict (dict): A dictionary of variables.
-        fun_control (dict): A dictionary of function control.
-    Returns:
-        list A list of dictionaries of hyper parameter values. Transformations are applied to the values.
-    Examples:
-        >>> import numpy as np
-            var_dict = {'a': np.array([1]),
-                        'b': np.array([2])}
-            fun_control = {'var_type': ['int', 'int']}
-            return_conf_list_from_var_dict(var_dict, fun_control)
-            var_dict = {'a': np.array([1, 3, 5]), 'b': np.array([2, 4, 6])}
-            fun_control = {'var_type': ['int', 'int']}
-            return_conf_list_from_var_dict(var_dict, fun_control)
-            {'a': [1, 3, 5], 'b': [2, 4, 6]}
-
-    """
-    conf_list = []
-    for values in iterate_dict_values(var_dict):
-        values = convert_keys(values, fun_control["var_type"])
-        values = get_dict_with_levels_and_types(fun_control=fun_control, v=values)
-        values = transform_hyper_parameter_values(fun_control=fun_control, hyper_parameter_values=values)
-        conf_list.append(values)
-    return conf_list
-
-
-def add_core_model_to_fun_control(core_model, fun_control, hyper_dict, filename) -> dict:
+def add_core_model_to_fun_control(core_model, fun_control, hyper_dict, filename=None) -> dict:
     """Add the core model to the function control dictionary.
     Args:
         core_model (class): The core model.
@@ -380,7 +599,7 @@ def add_core_model_to_fun_control(core_model, fun_control, hyper_dict, filename)
     if filename is None:
         new_hyper_dict = hyper_dict().load()
     else:
-        with open("river_hyper_dict.json", "r") as f:
+        with open(filename, "r") as f:
             new_hyper_dict = json.load(f)
     hyper_dict().load()
     fun_control.update({"core_model_hyper_dict": new_hyper_dict[core_model.__name__]})
@@ -394,10 +613,18 @@ def get_one_core_model_from_X(X, fun_control=None):
     return core_model
 
 
+def get_one_config_from_X(X, fun_control=None):
+    var_dict = assign_values(X, fun_control["var_name"])
+    config = return_conf_list_from_var_dict(var_dict, fun_control)[0]
+    return config
+
+
 def get_one_sklearn_model_from_X(X, fun_control=None):
     core_model = get_one_core_model_from_X(X=X, fun_control=fun_control)
     if fun_control["prep_model"] is not None:
         model = make_pipeline(fun_control["prep_model"], core_model)
+    else:
+        model = core_model
     return model
 
 
@@ -405,21 +632,23 @@ def get_one_river_model_from_X(X, fun_control=None):
     core_model = get_one_core_model_from_X(X=X, fun_control=fun_control)
     if fun_control["prep_model"] is not None:
         model = compose.Pipeline(fun_control["prep_model"], core_model)
+    else:
+        model = core_model
     return model
 
 
-def get_default_hyperparameters_for_core_model(fun_control, hyper_dict) -> dict:
-    X0 = get_default_hyperparameters_for_fun(fun_control, hyper_dict)
-    var_dict = assign_values(X0, fun_control["var_name"])
-    values = return_conf_list_from_var_dict(var_dict, fun_control)[0]
-    return values
-
-
-def get_default_hyperparameters_for_fun(fun_control, hyper_dict) -> np.array:
+def get_default_hyperparameters_as_array(fun_control, hyper_dict) -> np.array:
     X0 = get_default_values(fun_control)
-    river_hyper_dict_default = hyper_dict().load()
-    X0 = replace_levels_with_positions(river_hyper_dict_default[fun_control["core_model"].__name__], X0)
+    X0 = replace_levels_with_positions(hyper_dict[fun_control["core_model"].__name__], X0)
     X0 = get_values_from_dict(X0)
     X0 = np.array([X0])
     X0.shape[1]
     return X0
+
+
+def get_default_hyperparameters_for_core_model(fun_control) -> dict:
+    values = get_default_values(fun_control)
+    values = get_dict_with_levels_and_types(fun_control=fun_control, v=values)
+    values = convert_keys(values, fun_control["var_type"])
+    values = transform_hyper_parameter_values(fun_control=fun_control, hyper_parameter_values=values)
+    return values
